from Interface import Interface
from Layer import ILayer
import ZODB
from ZODB.PersistentMapping import PersistentMapping

class IStorage(Interface):
    """Abstraction around the storage of field level data"""
    def get(name, instance, **kwargs):
        """lookup a value for a given instance stored under 'name'"""

    def set(name, instance, value, **kwargs):
        """set a value under the key 'name' for retrevial by/for
        instance"""


class AttributeStorage:
    __implements__ = IStorage

    def get(self, name, instance, **kwargs):
        return getattr(instance, name)

    def set(self, name, instance, value, **kwargs):
        setattr(instance, name, value)
        instance._p_changed = 1

class ObjectManagedStorage:
    __implements__ = IStorage
    def get(self, name, instance, **kwargs):
        return instance._getOb(name)

    def set(self, name, instance, value, **kwargs):
        instance._setObject(name, value)
        instance._p_changed = 1

class MetadataStorage:
    __implements__ = (IStorage, ILayer)
    
    def initalizeInstance(self, instance):
        if not hasattr(instance, "_md"):
            instance._md = PersistentMapping()
            
    def get(self, name, instance, **kwargs):
        return instance._md[name]

    def set(self, name, instance, value, **kwargs):
        instance._md[name] = value
        instance._p_changed = 1
        
